/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Urban Rage Software.
The Initial Developer of the Original Code is Urban Rage Software.

Portions created by Urban Rage Software are
Copyright (C) 2000-2001 Urban Rage Software.
All Rights Reserved.

Contributor(s): Eric Plaster, Becki Sanford


RDFTree API

       RDFTree.js

Restrictions:
  This class is intended to make it easier to control a tree that has rdf datasources.  To use this class,
  you will need to follow a few simple rules:
    (1) Your tree must have an 'id'.  This is needed to find the tree and change it's attributes.
    (2) You must supply an 'id' for your 'treecol'.
    (3) Your 'treecell' under your 'treehead' must have the same 'id' as your 'treecol'+"Header".
        (i.e. your treecol has an id="linksColumn" and your treehead->treerow->treecell has
        an id="linksColumnHeader")
    (4) Must be an rdf tree.
    (5) Only supports single selection trees at this time.

Function List

    // Constructor
    RDFTree(aID)                              // creates the Tree object and gets the tree from the ID

    (get) tree                                // tree object
    addDataSource(aRDF_URL_Path)              // adds a datasource to the tree.  Must be a valid url path (i.e.
                                                 file:///tmp/bob.rdf)
    rebuild()                                 // Rebuilds the tree (resets it.  You need to do this when you
                                                 delete an entry in an rdf file)
    (get) selected                            // returns the currently selected index.
    (set) selected(which)                     // set the selected index.
    (get) children                            // returns the tree children
    (get) count                               // returns the number of items in the tree
    (get) treeitems                           // returns an array of treeitem elements
    (get) treecells                           // returns an array of treecell elements
    (get) selectedID                          // returns the 'id' of the selected item
    (get) selection                           // returns an array of selected treeitems
    (get) selectCount                         // returns the number of selected treeitems
    clearSelection(which)                     // clears item selections; if which = null, all selections are cleared
    getRowIndexOf(which)                      // gets the index of the given treeitem 'id'
    doSort(column)                            // sorts the given column (see Notes)
    refreshSort()                             // sorts to the previous settings (see Notes)
    sortColumn(column, sortKey[, direction])  // sorts the given column 'id' either 'ascending' or 'descending'
    sortToPreviousSettings()                  // sorts to the previous settings.

    // internal members.
    sort(column, Key, direction)              // sorts the given column 'id' either 'ascending' or 'descending'
    updateSortIndicator(column, direction)    // Updates the sort indicator

 Instructions:

       Warning: these API's are not for religious types


 Notes:
 (1) The doSort() and refreshSort() functions require the <treecell> to contain an observes 
     attribute that points to the <treecol> that defines the resource attribute.
     The following is a skeleton code snippet:
     
    <tree>
      <template/>
      <treecolgroup>
        <treecol id="fooName" resource="http://home.netscape.com/NC-rdf#fooName"/>
      </treecolgroup>
      <treehead>
        <treerow>
          <treecell value="Name" observes="fooName"
  		              class="treecell-inset-header sortDirectionIndicator"
  		              onclick="gShell.treeUtils.doSort('fooName')"/>
        </treerow>
      </treehead>
    </tree>

************/

/****************** Globals **********************/

const JSLIB_TREE                     = "tree.js";
const JSTREE_OK                      = true;
const JSLIB_TREE_SORT_PROGID         = '@mozilla.org/xul/xul-sort-service;1';

/****************** Globals **********************/


/****************** File Object Class *********************/

function RDFTree(aID) {
  if(aID)
    this.mID= aID;
} // constructor

RDFTree.prototype  = 
{
  mID           : null,
  mTree         : null,

  get tree()
    {
      if (this.mTree) return this.mTree;
      var tree = document.getElementById(this.mID);
      if(!tree) {
        throw("Unable to get tree.  ID: "+this.mID);
      }
      this.mTree = tree;
      return tree;
    },

  addDataSource :
    function(aRDF_URL_Path)
    {
      var tree = this.tree;

      var ds = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService().QueryInterface(Components.interfaces.nsIRDFService).GetDataSource(aRDF_URL_Path);

      tree.database.AddDataSource(ds);
      tree.builder.rebuild();
    },
    
  removeDataSource :
    function(aRDF_URL_Path)
    {
      var tree = this.tree;

      var ds = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService().QueryInterface(Components.interfaces.nsIRDFService).GetDataSource(aRDF_URL_Path);

      tree.database.RemoveDataSource(ds);
      tree.builder.rebuild();
    },
  
  focus :
    function()
    {
      // Force the button states to be updated by giving the tree focus
      // (this will also allow the user to immediately use the up/down
      // arrows --- bonus points!)
      var tree = this.tree;
      tree.focus();
    },

  rebuild :
    function()
    {
      this.sortToPreviousSettings();

      // Force the button states to be updated by giving the tree focus
      // (this will also allow the user to immediately use the up/down
      // arrows --- bonus points!)
      var tree = this.tree;
      tree.focus();
    },

  get selected()
    {
      var tree = this.tree;
      var select_list = tree.selectedItems;
      if ((select_list) && select_list.length > 0) {
        return this.getRowIndexOf(select_list[0].getAttribute('id'));
      }
      return null;
    },

  get selectCount()
    {
      return this.tree.selectedItems.length;
    },

  set selected(index)
    {
      var tree = this.tree;
      var body = this.children;

      if ( tree && body )	{
        var treeitems = body.getElementsByTagName('treeitem');
        if ( treeitems && treeitems.length > 0 ) {
          if (index >= treeitems.length) index = treeitems.length - 1;
          tree.focus();
          tree.selectItem(treeitems[index]);
        }
      }

      // Give the tree focus (this will also allow the user to immediately
      // use the up/dopwn arrows -- bonus points!)
      tree.focus();
    },

  get children()
    {
      var tree = this.tree;
      if ( tree && tree.childNodes ) {
        for ( var index = tree.childNodes.length - 1; index >= 0; index-- ) {
          if ( tree.childNodes[index].tagName == 'treechildren' ) {
            return (tree.childNodes[index]);
          }
        }
      }
      return null;
    },

  get count()
    {
      var children = this.children;
      if (children) {
        return children.getElementsByTagName('treeitem').length;
      }
      return 0;
    },

  get treeitems()
    {
      var children = this.children;
      if ( children )  {
        return (children.getElementsByTagName('treeitem'));
      }
    },

  get treecells()
    {
      var children = this.children;
      if ( children )  {
        return (children.getElementsByTagName('treecell'));
      }
    },

  get selectedID()
    {
      var tree = this.tree;
      var select_list = tree.selectedItems;
      if ((!select_list) || select_list.length < 1) return null;
      return select_list[0].getAttribute('id');
    },
  
  get selection()
    {
      return this.tree.selectedItems;
    },
  
  clearSelection :
    function(index)
    {
      var tree = this.tree;
      
      if ( tree ) {
        if (index == null) {
          tree.clearItemSelection();
          return;
        }

        var body = this.children;
        if ( body )	{
          var treeitems = body.getElementsByTagName('treeitem');
          if ( treeitems && treeitems.length > 0 ) {
            if ((index < 0) || (index > treeitems.length)) return;
            tree.focus();
            tree.removeItemFromSelection(treeitems[index]);
          }
        }
      }
    },

  getRowIndexOf :
    function(which)
    {
      var tree = this.tree;
      var body = this.children;

      if ( tree && body )	{
        var treeitems = body.getElementsByTagName('treeitem');
        if ( treeitems && treeitems.length > 0 )
        {
          for ( var i=0; i < treeitems.length; i++ ) {
            if (which == treeitems[i].getAttribute('id')) return i;
          }
        }
      }
      return 0;
    },

  //------------------------------------------------------------------------
  // Column sort utilities
  //------------------------------------------------------------------------
  doSort :
    function(column)
    {
      var tree = this.tree;
      var node = document.getElementById(column);
      if (!node) return false;

      var sortKey = node.getAttribute('resource');
      if (!sortKey) return false;

      var sortDirection = "ascending";
      if ( node.getAttribute('sortDirection') == "ascending" )
        sortDirection = "descending";
      else
        sortDirection = "ascending";

      this.sort(column, sortKey, sortDirection);

      // Save the current sort settings for use by SortToPreviousSettings()
      tree.setAttribute('sortColumn',column);
      tree.setAttribute('sortKey',sortKey);
      tree.setAttribute('sortDirection',sortDirection);
      node.setAttribute('sortDirection',sortDirection);

      return true;
    },

  //------------------------------------------------------------------------
  refreshSort :
    function()
    {
      // Retrieve the last sorted state from the tree element
      var tree = this.tree;
      var column = tree.getAttribute('sortColumn');
      var node = document.getElementById(column);
      var sortKey = node.getAttribute('resource');
      var sortDirection = node.getAttribute('sortDirection');

      this.sort(column, sortKey, sortDirection);

      // Force the button states to be updated by giving the tree focus
      // (this will also allow the user to immediately use the up/down
      // arrows --- bonus points!)
      tree.focus();
    },

  //------------------------------------------------------------------------
  sortColumn :
    function(column, sortKey, direction)
    {
      var tree = this.tree;
      var node = document.getElementById(column);
      if (!node) return false;

      // If a specific direction passed in as a parameter, use it; otherwise
      // get the last sort direction from the tree node (defaulting to
      // "ascending" if none available)
      var sortDirection;
      if (direction != null) {
        sortDirection = direction;
      }
      else {
        sortDirection = "ascending";
        if ( node.getAttribute('sortDirection') == "ascending" )
          sortDirection = "descending";
        else
          sortDirection = "ascending";
      }

      this.updateSortIndicator(column, sortDirection);
      this.sort(column, sortKey, sortDirection);


      // Save the current sort settings for use by SortToPreviousSettings()
      tree.setAttribute('sortColumn',column);
      tree.setAttribute('sortKey',sortKey);
      tree.setAttribute('sortDirection',sortDirection);
      node.setAttribute('sortDirection',sortDirection);

      return true;
    },

  //------------------------------------------------------------------------
  sort :
    function(column, key, direction)
    {
      var xulSortService = Components.classes[JSLIB_TREE_SORT_PROGID].getService();
      xulSortService = xulSortService.QueryInterface(Components.interfaces.nsIXULSortService);
      if (!xulSortService) return (false);

      var node = document.getElementById(column);
      if ( node )
      {
        // Save off the current item selection
        var tree = this.tree;
        var saveSelection;
        var selection = tree.selectedItems;
        if (selection.length > 0) saveSelection = selection[0].getAttribute('id');

        try {
          xulSortService.Sort(node, key, direction);
        }
        catch (ex) {}
        tree.clearItemSelection();

        // Restore the original item selection now that the sore is
        // complete
        if (saveSelection)
        {
          var rowElement = document.getElementById(saveSelection);
          tree.addItemToSelection(rowElement);
        }
      }
    },

  //------------------------------------------------------------------------
  sortToPreviousSettings :
    function()
    {
      var tree = this.tree;

      // Retrieve the last sorted state from the tree element
      var column = tree.getAttribute('sortColumn');	
      var sortKey = tree.getAttribute('sortKey');
      var sortDirection = tree.getAttribute('sortDirection');

      // Provide default sort data if none was d			
      if ( !column || !sortKey )
      {
          return;
      }
      if ( !sortDirection )
        sortDirection = 'ascending';

      this.updateSortIndicator(column,sortDirection);
      this.sort(column, sortKey, sortDirection);
    },

  //------------------------------------------------------------------------
  // Sets the column header sort icon based on the requested 
  // column and direction.
  // 
  // Notes:
  // (1) This function relies on the first part of the 
  //     <treecell id> matching the <treecol id>.  The treecell
  //     id must have a "Header" suffix.
  // (2) By changing the "sortDirection" attribute, a different
  //     CSS style will be used, thus changing the icon based on
  //     the "sortDirection" parameter.
  //------------------------------------------------------------------------
  updateSortIndicator :
    function(column,sortDirection)
    {
      // Find the <treerow> element
      var treerow = document.getElementById('headRow');
      var id = column + "Header";

      if (treerow)
      {
        // Grab all of the <treecell> elements
        var treecell = treerow.getElementsByTagName('treecell');
        if (treecell)
        {
          // Loop through each treecell...
          var node_count = treecell.length;
          for (var i=0; i < node_count; i++)
          {
            // Is this the requested column ?
            if (id == treecell[i].getAttribute('id'))
            {
              // Set the sortDirection so the class (CSS) will add the
              // appropriate icon to the header cell
              treecell[i].setAttribute('sortDirection',sortDirection);
            }
            else
            {
              // This is not the sorted row
              treecell[i].removeAttribute('sortDirection');
            }
          }
        }
      }
    }
};

